﻿// hiddemo.cpp : 此文件包含 "main" 函数。程序执行将在此处开始并结束。
//

#include <iostream>
#include "libusb.h"

static int verbose = 0;
/* Ctrl Message type */
enum cmd_ctrl_msg_type
{
	/* 0 Reserved */
	CMD_SYNC = 1,
	CMD_CONNECT,
	CMD_DISCONNECT,
	CMD_RESET,
	CMD_ACCEPT,
	CMD_REJECT,
	CMD_FINISHED,
	CMD_JUMP_APROM,
	CMD_JUMP_DFU,
	CMD_GET_STATUS,
	CMD_ERROR,
	/*app*/
	CMD_GET_DATA,
	CMD_GET_FILE
};

/* Data Message type */
enum cmd_data_msg_type
{
	/* 0 - 63 Reserved */
	CMD_HEAD = 64,
	CMD_PUT_DATA,//
};


/* attribute type */
enum attribute_data_type
{
	/* 0 - 63 Reserved */
	ATT_ADC = 0x001,
	ATT_ADC_QUEUE = 0x002,
	ATT_ADC_QUEUE_10K = 0x004, //10K数据
	ATT_SETTINGS = 0x008,
	ATT_PD_PACKET = 0x010,
	ATT_PD_STATUS = 0x020,
	ATT_QC_PACKET = 0x040,
	ATT_TICK = 0x080,
	ATT_TICK__ = 0x100,
};


typedef struct {
	libusb_device_handle * handle;
	struct libusb_endpoint_descriptor hidendpoint;
	struct libusb_endpoint_descriptor user_endpoint;
	uint8_t hid_in;
	uint8_t hid_out;
	uint8_t user_in;
	uint8_t user_out;
}my_USB_Struct;

typedef union
{
	uint32_t object;
	uint16_t word[2];
	uint8_t byte[4];
	struct
	{
		uint32_t type : 7;        /*!< 大于63为数据                    */
		uint32_t extend : 1;        /*!< 用于传输超大数据包              */
		uint32_t id : 8;        /*!< id 不停的累加                   */
		uint32_t : 1;        /*!< 数据为加密格式                 */
		uint32_t att : 15;       /*!< 属性代码    */
	}ctrl;
	struct
	{
		uint32_t type : 7;
		uint32_t extend : 1;
		uint32_t id : 8;
		uint32_t encode : 1;
		uint32_t permi : 2;
		uint32_t : 3;
		uint32_t obj : 10;       /*!<  4字节一个对象   */
	}data;
	struct
	{
		uint32_t att : 15;
		uint32_t next : 1;
		uint32_t chunk : 6;
		uint32_t size : 10;       /*!<  不能超过4095-8个字节   CDC HID 小于 64字节*/
	}header;
}MsgHeader_TypeDef;

typedef struct {
	int32_t         Vbus;
	int32_t         Ibus;
	int32_t         Vbus_avg;
	int32_t         Ibus_avg;
	int32_t         Vbus_ori_avg;
	int32_t         Ibus_ori_avg;
	int16_t         Temp;
	uint16_t        Vcc1;
	uint16_t        Vcc2;
	uint16_t        Vdp;
	uint16_t        Vdm;
	uint16_t        Vdd;
	uint8_t         Rate : 2;
	uint8_t         n[3];
}AdcData_TypeDef;


static ssize_t my_get_device_number(libusb_device **dev, uint16_t idVendor)
{
	struct libusb_device_descriptor desc;
	ssize_t cnt = 0;
	int ret, i;

	for (i = 0; dev[i]; ++i) {
		ret = libusb_get_device_descriptor(dev[i], &desc);
		if (ret < 0) {
			fprintf(stderr, "failed to get device descriptor");
			continue;
		}
		if ((desc.idVendor == idVendor) || (desc.idVendor == 0x5558))
		{
			cnt++;
		}
	}
	return cnt;
}

static int my_open_device_with_vid_pid(my_USB_Struct *dev, uint16_t vendor_id, uint16_t product_id, uint16_t index)
{
	struct libusb_device **devs;
	struct libusb_device *found = NULL;
	struct libusb_device *device;
	struct libusb_device_handle *dev_handle = NULL;
	struct libusb_config_descriptor *conf_desc;
	const struct libusb_endpoint_descriptor *endpoint;
	int nb_ifaces, first_iface = -1;
	int i = 0, j, k, r, cnt = 0;
	uint8_t hid_endpoint_in = 0, hid_endpoint_out = 0;
	uint8_t user_endpoint_in = 0, user_endpoint_out = 0;

	if (libusb_get_device_list(NULL, &devs) < 0)
		goto out;
	r = my_get_device_number(devs, vendor_id);
	if (r < index)
		goto out;

	while ((device = devs[i++]) != NULL) {
		struct libusb_device_descriptor desc;
		r = libusb_get_device_descriptor(device, &desc);
		if (r < 0)
			goto out;
		if (desc.idVendor == vendor_id) {
			if (product_id || desc.idProduct == product_id) {
				if (cnt != index) {
					cnt++;
					continue;
				}
			}
			found = device;
			break;
		}
	}

	if (found) {
		r = libusb_open(found, &dev_handle);
		if (r < 0)
			goto out;

		libusb_set_auto_detach_kernel_driver(dev_handle, 1);

		libusb_get_config_descriptor(device, 0, &conf_desc);
		nb_ifaces = conf_desc->bNumInterfaces;
		if (nb_ifaces > 0)
			first_iface = conf_desc->interface[0].altsetting[0].bInterfaceNumber;

		for (i = 0; i < nb_ifaces; i++)
		{
			r = libusb_claim_interface(dev_handle, i);
			if (r != LIBUSB_SUCCESS) {
				continue;
			}
			for (j = 0; j < conf_desc->interface[i].num_altsetting; j++) {
				for (k = 0; k < conf_desc->interface[i].altsetting[j].bNumEndpoints; k++) {
					struct libusb_ss_endpoint_companion_descriptor *ep_comp = NULL;
					endpoint = &conf_desc->interface[i].altsetting[j].endpoint[k];
					printf("       endpoint[%d].address: %02X\n", k, endpoint->bEndpointAddress);
					// Use the first interrupt or bulk IN/OUT endpoints as default for testing
					if ((endpoint->bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) == (LIBUSB_TRANSFER_TYPE_INTERRUPT)) {
						if (endpoint->bEndpointAddress & LIBUSB_ENDPOINT_IN) {
							if (!hid_endpoint_in)
								hid_endpoint_in = endpoint->bEndpointAddress;
						}
						else {
							if (!hid_endpoint_out)
								hid_endpoint_out = endpoint->bEndpointAddress;
						}
					}
					else if ((endpoint->bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) == (LIBUSB_TRANSFER_TYPE_BULK)) {
						if (endpoint->bEndpointAddress & LIBUSB_ENDPOINT_IN) {
							if (!user_endpoint_in)
								user_endpoint_in = endpoint->bEndpointAddress;
						}
						else {
							if (!user_endpoint_out)
								user_endpoint_out = endpoint->bEndpointAddress;
						}
					}
				}
			}
		}

		libusb_free_config_descriptor(conf_desc);

	}
	dev->handle = dev_handle;
	dev->hid_in = hid_endpoint_in;
	dev->hid_out = hid_endpoint_out;
	dev->user_in = user_endpoint_in;
	dev->user_out = user_endpoint_out;
out:
	libusb_free_device_list(devs, 1);
	return LIBUSB_SUCCESS;
}

static int  my_hid_get_data(my_USB_Struct *dev, uint8_t *rbuf, int * size)
{
	uint8_t tbuf[64];
	MsgHeader_TypeDef head;
	int ret, txsize;
	head.object = 0;
	head.ctrl.type = CMD_GET_DATA;
	head.ctrl.att = ATT_ADC;

	memcpy(tbuf, &head, sizeof head);

	if (rbuf) {
		ret = libusb_interrupt_transfer(dev->handle, dev->hid_out, tbuf, sizeof(tbuf), &txsize, 1000);
		if (ret != LIBUSB_SUCCESS)
			return ret;
		ret = libusb_interrupt_transfer(dev->handle, dev->hid_in, rbuf, 64, &txsize, 1000);
		if (ret != LIBUSB_SUCCESS)
			return ret;
		*size = txsize;
		return LIBUSB_SUCCESS;
	}
	return LIBUSB_ERROR_INVALID_PARAM;
}

static void test(void)
{
	my_USB_Struct dev;
	my_USB_Struct dev2;
	uint8_t rbuf[64];
	int ret, i = 2000, size;
	AdcData_TypeDef value;
	dev.handle = 0;
	my_open_device_with_vid_pid(&dev, 0x5FC9, 0x0000, 0);
	//my_open_device_with_vid_pid(&dev2, 0x5FC9, 0x0061, 1);


	if (dev.handle) {

		while (i--) {
			if (my_hid_get_data(&dev, rbuf, &size) == LIBUSB_SUCCESS) {
				MsgHeader_TypeDef head;
				MsgHeader_TypeDef head_ext;
				memcpy(&head, rbuf, sizeof(head));
				memcpy(&head_ext, rbuf + sizeof(head), sizeof(head));
				memcpy(&value, rbuf + sizeof(head)*2, sizeof(value));
				if (head.data.type == CMD_PUT_DATA && head_ext.header.att == ATT_ADC)
				{
					printf("Vbus: %fV,IBus: %fA\n", (double)value.Vbus_avg / 1000000, (double)value.Ibus_avg / 1000000);
					printf(" Vdp: %fV,Vdm : %fV\n"
						   "Vcc1: %fV,Vcc2: %fV", (double)value.Vdp / 10000, (double)value.Vdm / 10000
						, (double)value.Vcc1 / 10000, (double)value.Vcc2 / 10000);
				}
			}
			//if (dev2.handle) {
			//	if (my_hid_get_data(&dev2, rbuf, &size) == LIBUSB_SUCCESS) {
			//		memcpy(&value, rbuf + 6, sizeof(value));
			//		//system("cls");
			//		printf("\nVbus: %04fV  Iload: %04fA", (double)value.Voltage / 1000000, (double)value.Current / 1000000);
			//	}
			//}
			Sleep(150);
			system("cls");
		}
	}
}
int main(int argc, char *argv[])
{
	libusb_device **devs;

	ssize_t cnt;
	int r, i;

	if (argc > 1 && !strcmp(argv[1], "-v"))
		verbose = 1;

	const struct libusb_version* version;
	version = libusb_get_version();
	printf("Using libusb v%d.%d.%d.%d\n\n", version->major, version->minor, version->micro, version->nano);
	r = libusb_init(NULL);
	if (r < 0)
		return r;

	test();

	libusb_exit(NULL);
	return 0;
}

// 运行程序: Ctrl + F5 或调试 >“开始执行(不调试)”菜单
// 调试程序: F5 或调试 >“开始调试”菜单

// 入门使用技巧: 
//   1. 使用解决方案资源管理器窗口添加/管理文件
//   2. 使用团队资源管理器窗口连接到源代码管理
//   3. 使用输出窗口查看生成输出和其他消息
//   4. 使用错误列表窗口查看错误
//   5. 转到“项目”>“添加新项”以创建新的代码文件，或转到“项目”>“添加现有项”以将现有代码文件添加到项目
//   6. 将来，若要再次打开此项目，请转到“文件”>“打开”>“项目”并选择 .sln 文件
